==> verttice_ui-0.0.153.tar.gz: lib/src/widgets/data_factory/form_factory/address_on_map_form.dart <==
// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import "package:collection/collection.dart";
import "package:flutter_typeahead/flutter_typeahead.dart";
import 'package:flutter_google_places_sdk/flutter_google_places_sdk.dart';
import 'package:verttice_models/verttice_models.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class AddressOnMapFormFactory extends StatefulWidget {
  final String displayName;
  final String name;
  final Map<String, dynamic>? value;
  final Function(String name, dynamic value) onChanged;
  final FormFieldState<dynamic>? state;
  final String? tip;

  const AddressOnMapFormFactory({
    Key? key,
    required this.displayName,
    required this.name,
    required this.onChanged,
    this.state,
    this.tip,
    this.value,
  }) : super(key: key);

  @override
  State createState() => _AddressOnMapFormFactoryState();
}

class _AddressOnMapFormFactoryState extends State<AddressOnMapFormFactory> {
  late Map<String, dynamic>? _value;

  @override
  void initState() {
    super.initState();
    _value = widget.value;
  }

  @override
  void didUpdateWidget(AddressOnMapFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    final eq = const MapEquality().equals;

    if (!eq(
      _value,
      widget.value,
    )) {
      _value = widget.value;
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      tip: widget.tip,
      onClear: () {
        widget.onChanged(widget.name, null);
        widget.state?.didChange(null);
      },
      child: Tooltip(
        message: _value?["fullAddress"] ?? "",
        child: SelectableText(
          _value?["fullAddress"] ?? "",
          maxLines: 3,
        ),
      ),
      onPressed: () async {
        fullPageDialog(
          context: context,
          builder: (BuildContext context) => SearchPlace(
            address: _value != null ? AddressOnMapModel.fromMap(_value!) : null,
            onSelect: (AddressOnMapModel data) {
              final addressOnMapData = data.toMap();
              widget.onChanged(widget.name, addressOnMapData);
              widget.state?.didChange(addressOnMapData);
            },
          ),
        );
      },
    );
  }
}

class SearchPlace extends StatefulWidget {
  final Function(AddressOnMapModel value) onSelect;
  final AddressOnMapModel? address;
  const SearchPlace({
    Key? key,
    required this.onSelect,
    this.address,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _SearchPlaceState();
}

class _SearchPlaceState extends State<SearchPlace> {
  late TextEditingController _controller;
  // bool _isSearchByAddress = true;
  AddressOnMapModel? _chosenPlace;

  late final FlutterGooglePlacesSdk _places;
  final List<String> _countries = ["uk"];
  final PlaceTypeFilter _placeTypeFilter = PlaceTypeFilter.ALL;
  final List<PlaceField> _placeFields = [
    PlaceField.Address,
    PlaceField.AddressComponents,
    PlaceField.Id,
    PlaceField.Location,
    PlaceField.Name,
    PlaceField.PlusCode,
    PlaceField.UTCOffset,
    PlaceField.Viewport,
    PlaceField.WebsiteUri,
  ];

  @override
  void initState() {
    super.initState();

    _places = FlutterGooglePlacesSdk(VertticeUI.instance.firebaseApiKey);

    _chosenPlace = widget.address;

    _controller = TextEditingController.fromValue(
      TextEditingValue(
        text: _chosenPlace?.address ?? '',
      ),
    );
  }

  @override
  void didUpdateWidget(SearchPlace oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_chosenPlace?.latLng?.lat != widget.address?.latLng?.lat ||
        _chosenPlace?.latLng?.lng != widget.address?.latLng?.lng) {
      _chosenPlace = widget.address;

      _controller = TextEditingController.fromValue(
        TextEditingValue(
          text: _chosenPlace?.address ?? '',
        ),
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: ThemedAppBar(
        navigateBackButton: true,
        actions: [
          if (_chosenPlace != null)
            Tooltip(
              message: "Select a place",
              child: ThemedIconButton(
                onPressed: () {
                  widget.onSelect(_chosenPlace ?? const AddressOnMapModel());

                  Navigator.of(context).pop();
                },
                icon: ThemedIcons.check(),
              ),
            ),
        ],
        title: "Find a place",
      ),
      body: Padding(
        padding: const EdgeInsets.all(30),
        child: LayoutBuilder(
          builder: (context, constraints) {
            final isMobile = ScreenSizeUtils.isMobile(context: context);
            const double dividerWidth = 2;
            final double? mobileSectionHeight = isMobile ? 230 : null;
            const double mobileMapHeight = 550;
            final double? desktopSectionWidth =
                isMobile ? null : (constraints.maxWidth / 2) - dividerWidth;
            return SingleChildScrollView(
              controller: ScrollController(),
              child: Flex(
                direction: ScreenSizeUtils.isMobile(context: context)
                    ? Axis.vertical
                    : Axis.horizontal,
                crossAxisAlignment: CrossAxisAlignment.start,
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  SizedBox(
                    width: desktopSectionWidth,
                    height: mobileSectionHeight,
                    child: Padding(
                      padding: const EdgeInsets.all(16.0),
                      child: Column(
                        children: [
                          SizedBox(
                            height: 100,
                            child: Center(
                              child: TextFormFactory(
                                value: _chosenPlace?.addressPrefix ?? '',
                                displayName:
                                    'Property prefix (e.g. flat, property name)',
                                name: 'placePrefix',
                                onChanged: (String name, dynamic data) {
                                  final currentPlace =
                                      _chosenPlace ?? const AddressOnMapModel();
                                  setState(() {
                                    _chosenPlace = currentPlace.copyWith(
                                      fullAddress:
                                          "$data${data == '' ? '' : ' '}${currentPlace.address ?? ""}",
                                      addressPrefix: data,
                                    );
                                  });
                                },
                                showStandardBorder: false,
                              ),
                            ),
                          ),
                          TypeAheadField(
                            textFieldConfiguration: TextFieldConfiguration(
                              controller: _controller,
                              maxLines: 2,
                              decoration: const InputDecoration(
                                border: UnderlineInputBorder(),
                                label: Text(
                                  "Search a place on maps",
                                ),
                              ),
                            ),
                            suggestionsCallback: (pattern) async {
                              List<AutocompletePrediction> predictions = [];
                              if (pattern == "") {
                                return predictions;
                              }
                              final result =
                                  await _places.findAutocompletePredictions(
                                pattern,
                                countries: _countries,
                                placeTypeFilter: _placeTypeFilter,
                                newSessionToken: false,
                              );
                              return result.predictions;
                            },
                            itemBuilder: (
                              context,
                              AutocompletePrediction suggestion,
                            ) {
                              return ListTile(
                                leading: ThemedIcons.pin(),
                                title: Text(suggestion.fullText),
                                subtitle: Text(
                                  "${suggestion.primaryText} - ${suggestion.secondaryText}",
                                ),
                              );
                            },
                            onSuggestionSelected:
                                (AutocompletePrediction suggestion) async {
                              final result = await _places.fetchPlace(
                                suggestion.placeId,
                                fields: _placeFields,
                              );
                              setState(() {
                                final currentPlace =
                                    _chosenPlace ?? const AddressOnMapModel();
                                final address = result.place?.addressComponents
                                    ?.map((e) => e.name)
                                    .toList()
                                    .join(" ");
                                _chosenPlace = currentPlace.copyWith(
                                  fullAddress:
                                      "${currentPlace.addressPrefix ?? ""}${currentPlace.addressPrefix != null ? " " : ""}${address ?? ""}",
                                  address: address,
                                  latLng: result.place?.latLng != null
                                      ? LatLngModel.fromMap(
                                          result.place!.latLng!.toMap())
                                      : null,
                                  name: result.place?.name,
                                  utcOffsetMinutes:
                                      result.place?.utcOffsetMinutes,
                                  viewport: result.place?.viewport != null
                                      ? LatLngBoundsModel.fromMap(
                                          result.place!.viewport!.toMap())
                                      : null,
                                  websiteUri:
                                      result.place?.websiteUri?.toString(),
                                );
                                _controller = TextEditingController.fromValue(
                                  TextEditingValue(
                                    text: _chosenPlace?.address ?? '',
                                  ),
                                );
                              });
                            },
                          )
                        ],
                      ),
                    ),
                  ),
                  if (_chosenPlace?.latLng?.lat != null &&
                      _chosenPlace?.latLng?.lng != null) ...[
                    if (isMobile)
                      Padding(
                        padding: const EdgeInsets.all(8.0),
                        child: ThemedDivider(
                          width: constraints.maxWidth,
                          height: dividerWidth,
                        ),
                      )
                    else
                      ThemedDivider(
                        width: dividerWidth,
                        height: constraints.maxHeight,
                      ),
                    SizedBox(
                      width: desktopSectionWidth,
                      height: isMobile
                          ? mobileMapHeight
                          : MediaQuery.of(context).size.height - 250,
                      child: Padding(
                        padding: const EdgeInsets.all(16.0),
                        child: Center(
                          child: Maps(
                            mapHeight: isMobile
                                ? mobileMapHeight
                                : MediaQuery.of(context).size.height - 250,
                            lat: _chosenPlace!.latLng!.lat,
                            lng: _chosenPlace!.latLng!.lng,
                            placeName:
                                "${_chosenPlace?.addressPrefix ?? ""}${_chosenPlace?.addressPrefix == null || _chosenPlace?.addressPrefix == '' ? '' : ' '}${_chosenPlace?.address ?? ""}",
                          ),
                        ),
                      ),
                    ),
                  ],
                ],
              ),
            );
          },
        ),
      ),
    );
  }
}

==> verttice_ui-0.1.2.tar.gz: lib/src/widgets/data_factory/form_factory/address_on_map_form.dart <==
// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import "package:collection/collection.dart";
import "package:flutter_typeahead/flutter_typeahead.dart";
import 'package:flutter_google_places_sdk/flutter_google_places_sdk.dart';
import 'package:verttice_types/verttice_types.dart';
// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class AddressOnMapFormFactory extends StatefulWidget {
  final String displayName;
  final String name;
  final Map<String, dynamic>? value;
  final Function(String name, dynamic value) onChanged;
  final FormFieldState<dynamic>? state;
  final String? tip;

  const AddressOnMapFormFactory({
    Key? key,
    required this.displayName,
    required this.name,
    required this.onChanged,
    this.state,
    this.tip,
    this.value,
  }) : super(key: key);

  @override
  State createState() => _AddressOnMapFormFactoryState();
}

class _AddressOnMapFormFactoryState extends State<AddressOnMapFormFactory> {
  late Map<String, dynamic>? _value;

  @override
  void initState() {
    super.initState();
    _value = widget.value;
  }

  @override
  void didUpdateWidget(AddressOnMapFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    final eq = const MapEquality().equals;

    if (!eq(
      _value,
      widget.value,
    )) {
      _value = widget.value;
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      tip: widget.tip,
      onClear: () {
        widget.onChanged(widget.name, null);
        widget.state?.didChange(null);
      },
      child: Tooltip(
        message: _value?["fullAddress"] ?? "",
        child: SelectableText(
          _value?["fullAddress"] ?? "",
          maxLines: 3,
        ),
      ),
      onPressed: () async {
        fullPageDialog(
          context: context,
          builder: (BuildContext context) => SearchPlace(
            address: _value != null ? AddressOnMapModel.fromMap(_value!) : null,
            onSelect: (AddressOnMapModel data) {
              final addressOnMapData = data.toMap();
              widget.onChanged(widget.name, addressOnMapData);
              widget.state?.didChange(addressOnMapData);
            },
          ),
        );
      },
    );
  }
}

class SearchPlace extends StatefulWidget {
  final Function(AddressOnMapModel value) onSelect;
  final AddressOnMapModel? address;
  const SearchPlace({
    Key? key,
    required this.onSelect,
    this.address,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _SearchPlaceState();
}

class _SearchPlaceState extends State<SearchPlace> {
  late TextEditingController _controller;
  // bool _isSearchByAddress = true;
  AddressOnMapModel? _chosenPlace;

  late final FlutterGooglePlacesSdk _places;
  final List<String> _countries = ["uk"];
  final PlaceTypeFilter _placeTypeFilter = PlaceTypeFilter.ALL;
  final List<PlaceField> _placeFields = [
    PlaceField.Address,
    PlaceField.AddressComponents,
    PlaceField.Id,
    PlaceField.Location,
    PlaceField.Name,
    PlaceField.PlusCode,
    PlaceField.UTCOffset,
    PlaceField.Viewport,
    PlaceField.WebsiteUri,
  ];

  @override
  void initState() {
    super.initState();

    _places = FlutterGooglePlacesSdk(VertticeUI.instance.firebaseApiKey);

    _chosenPlace = widget.address;

    _controller = TextEditingController.fromValue(
      TextEditingValue(
        text: _chosenPlace?.address ?? '',
      ),
    );
  }

  @override
  void didUpdateWidget(SearchPlace oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_chosenPlace?.latLng?.lat != widget.address?.latLng?.lat ||
        _chosenPlace?.latLng?.lng != widget.address?.latLng?.lng) {
      _chosenPlace = widget.address;

      _controller = TextEditingController.fromValue(
        TextEditingValue(
          text: _chosenPlace?.address ?? '',
        ),
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: ThemedAppBar(
        navigateBackButton: true,
        actions: [
          if (_chosenPlace != null)
            Tooltip(
              message: "Select a place",
              child: ThemedIconButton(
                onPressed: () {
                  widget.onSelect(_chosenPlace ?? const AddressOnMapModel());

                  Navigator.of(context).pop();
                },
                icon: ThemedIcons.check(),
              ),
            ),
        ],
        title: "Find a place",
      ),
      body: Padding(
        padding: const EdgeInsets.all(30),
        child: LayoutBuilder(
          builder: (context, constraints) {
            final isMobile = ScreenSizeUtils.isMobile(context: context);
            const double dividerWidth = 2;
            final double? mobileSectionHeight = isMobile ? 230 : null;
            const double mobileMapHeight = 550;
            final double? desktopSectionWidth =
                isMobile ? null : (constraints.maxWidth / 2) - dividerWidth;
            return SingleChildScrollView(
              controller: ScrollController(),
              child: Flex(
                direction: ScreenSizeUtils.isMobile(context: context)
                    ? Axis.vertical
                    : Axis.horizontal,
                crossAxisAlignment: CrossAxisAlignment.start,
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  SizedBox(
                    width: desktopSectionWidth,
                    height: mobileSectionHeight,
                    child: Padding(
                      padding: const EdgeInsets.all(16.0),
                      child: Column(
                        children: [
                          SizedBox(
                            height: 100,
                            child: Center(
                              child: TextFormFactory(
                                value: _chosenPlace?.addressPrefix ?? '',
                                displayName:
                                    'Property prefix (e.g. flat, property name)',
                                name: 'placePrefix',
                                onChanged: (String name, dynamic data) {
                                  final currentPlace =
                                      _chosenPlace ?? const AddressOnMapModel();
                                  setState(() {
                                    _chosenPlace = currentPlace.copyWith(
                                      fullAddress:
                                          "$data${data == '' ? '' : ' '}${currentPlace.address ?? ""}",
                                      addressPrefix: data,
                                    );
                                  });
                                },
                                showStandardBorder: false,
                              ),
                            ),
                          ),
                          TypeAheadField(
                            textFieldConfiguration: TextFieldConfiguration(
                              controller: _controller,
                              maxLines: 2,
                              decoration: const InputDecoration(
                                border: UnderlineInputBorder(),
                                label: Text(
                                  "Search a place on maps",
                                ),
                              ),
                            ),
                            suggestionsCallback: (pattern) async {
                              List<AutocompletePrediction> predictions = [];
                              if (pattern == "") {
                                return predictions;
                              }
                              final result =
                                  await _places.findAutocompletePredictions(
                                pattern,
                                countries: _countries,
                                placeTypeFilter: _placeTypeFilter,
                                newSessionToken: false,
                              );
                              return result.predictions;
                            },
                            itemBuilder: (
                              context,
                              AutocompletePrediction suggestion,
                            ) {
                              return ListTile(
                                leading: ThemedIcons.pin(),
                                title: Text(suggestion.fullText),
                                subtitle: Text(
                                  "${suggestion.primaryText} - ${suggestion.secondaryText}",
                                ),
                              );
                            },
                            onSuggestionSelected:
                                (AutocompletePrediction suggestion) async {
                              final result = await _places.fetchPlace(
                                suggestion.placeId,
                                fields: _placeFields,
                              );
                              setState(() {
                                final currentPlace =
                                    _chosenPlace ?? const AddressOnMapModel();
                                final address = result.place?.addressComponents
                                    ?.map((e) => e.name)
                                    .toList()
                                    .join(" ");
                                _chosenPlace = currentPlace.copyWith(
                                  fullAddress:
                                      "${currentPlace.addressPrefix ?? ""}${currentPlace.addressPrefix != null ? " " : ""}${address ?? ""}",
                                  address: address,
                                  latLng: result.place?.latLng != null
                                      ? LatLngModel.fromMap(
                                          result.place!.latLng!.toMap())
                                      : null,
                                  name: result.place?.name,
                                  utcOffsetMinutes:
                                      result.place?.utcOffsetMinutes,
                                  viewport: result.place?.viewport != null
                                      ? LatLngBoundsModel.fromMap(
                                          result.place!.viewport!.toMap())
                                      : null,
                                  websiteUri:
                                      result.place?.websiteUri?.toString(),
                                );
                                _controller = TextEditingController.fromValue(
                                  TextEditingValue(
                                    text: _chosenPlace?.address ?? '',
                                  ),
                                );
                              });
                            },
                          )
                        ],
                      ),
                    ),
                  ),
                  if (_chosenPlace?.latLng?.lat != null &&
                      _chosenPlace?.latLng?.lng != null) ...[
                    if (isMobile)
                      Padding(
                        padding: const EdgeInsets.all(8.0),
                        child: ThemedDivider(
                          width: constraints.maxWidth,
                          height: dividerWidth,
                        ),
                      )
                    else
                      ThemedDivider(
                        width: dividerWidth,
                        height: constraints.maxHeight,
                      ),
                    SizedBox(
                      width: desktopSectionWidth,
                      height: isMobile
                          ? mobileMapHeight
                          : MediaQuery.of(context).size.height - 250,
                      child: Padding(
                        padding: const EdgeInsets.all(16.0),
                        child: Center(
                          child: Maps(
                            mapHeight: isMobile
                                ? mobileMapHeight
                                : MediaQuery.of(context).size.height - 250,
                            lat: _chosenPlace!.latLng!.lat,
                            lng: _chosenPlace!.latLng!.lng,
                            placeName:
                                "${_chosenPlace?.addressPrefix ?? ""}${_chosenPlace?.addressPrefix == null || _chosenPlace?.addressPrefix == '' ? '' : ' '}${_chosenPlace?.address ?? ""}",
                          ),
                        ),
                      ),
                    ),
                  ],
                ],
              ),
            );
          },
        ),
      ),
    );
  }
}
